! (C) 2008 D. V. Wiebe
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! This file is part of the GetData project.
!
! This program is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by the
! Free Software Foundation; either version 2 of the License, or (at your
! option) any later version.
!
! GetData is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
! for more details.
!
! You should have received a copy of the GNU General Public License along
! with GetData; if not, write to the Free Software Foundation, Inc.,
! 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
!
! -*- F95 -*-

module getdata
implicit none


! Error codes
integer, parameter :: GD_E_OK=0
integer, parameter :: GD_E_OPEN=1
integer, parameter :: GD_E_FORMAT=2
integer, parameter :: GD_E_TRUNC=3
integer, parameter :: GD_E_CREAT=4
integer, parameter :: GD_E_BAD_CODE=5
integer, parameter :: GD_E_BAD_TYPE=6
integer, parameter :: GD_E_RAW_IO=7
integer, parameter :: GD_E_OPEN_INCLUDE=8
integer, parameter :: GD_E_INTERNAL_ERROR=9
integer, parameter :: GD_E_ALLOC=10
integer, parameter :: GD_E_RANGE=11
integer, parameter :: GD_E_OPEN_LINFILE=12
integer, parameter :: GD_E_RECURSE_LEVEL=13
integer, parameter :: GD_E_BAD_DIRFILE=14
integer, parameter :: GD_E_BAD_FIELD_TYPE=15
integer, parameter :: GD_E_ACCMODE=16
integer, parameter :: GD_E_UNSUPPORTED=17
integer, parameter :: GD_E_BAD_ENTRY=19
integer, parameter :: GD_E_DUPLICATE=20
integer, parameter :: GD_E_DIMENSION=21
integer, parameter :: GD_E_BAD_INDEX=22
integer, parameter :: GD_E_BAD_SCALAR=23
integer, parameter :: GD_E_BAD_REFERENCE=24
integer, parameter :: GD_E_PROTECTED=25

! Open flags
integer, parameter :: GD_RDONLY=0
integer, parameter :: GD_RDWR=1
integer, parameter :: GD_CREAT=16
integer, parameter :: GD_EXCL=32
integer, parameter :: GD_TRUNC=64
integer, parameter :: GD_BIG_ENDIAN=4
integer, parameter :: GD_LITTLE_ENDIAN=8
integer, parameter :: GD_FORCE_ENCODDED=2
integer, parameter :: GD_FORCE_ENDIAN=256
integer, parameter :: GD_PEDANTIC=128
integer, parameter :: GD_AUTO_ENCODED=0
integer, parameter :: GD_UNENCODED=4096
integer, parameter :: GD_TEXT_ENCODED=8192
integer, parameter :: GD_SLIM_ENCODED=12288

! Field types
integer, parameter :: GD_NO_ENTRY=0
integer, parameter :: GD_RAW_ENTRY=1
integer, parameter :: GD_LINCOM_ENTRY=2
integer, parameter :: GD_LINTERP_ENTRY=3
integer, parameter :: GD_BIT_ENTRY=4
integer, parameter :: GD_MULTIPLY_ENTRY=5
integer, parameter :: GD_PHASE_ENTRY=6
integer, parameter :: GD_INDEX_ENTRY=7
integer, parameter :: GD_CONST_ENTRY=257
integer, parameter :: GD_STRING_ENTRY=258

! Data types -- the unsigned types won't work when passed as
!               a return type, but we keep them anyways, since
!               they might appear as a result of calling fget_entry
integer, parameter :: GD_NULL=0
integer, parameter :: GD_UINT8=1
integer, parameter :: GD_INT8=17
integer, parameter :: GD_UINT16=2
integer, parameter :: GD_INT16=18
integer, parameter :: GD_UINT32=4
integer, parameter :: GD_INT32=20
integer, parameter :: GD_UINT64=8
integer, parameter :: GD_INT64=24
integer, parameter :: GD_FLOAT32=36
integer, parameter :: GD_FLOAT64=40


integer, parameter :: GD_FIELD_LEN=4096

type gd_entry
  integer :: field_type, n_fields, spf, data_type, bitnum, numbits, shift
  integer :: fragment_index
  character(len=GD_FIELD_LEN), dimension(3) :: field
  real*8, dimension(3) :: m, b
end type

contains
function fdirfile_open (dirfilename, flags)
  integer :: fdirfile_open
  character (len=*), intent (in) :: dirfilename
  integer, intent (in) :: flags

  ! call f77 library
  call gdfopn(fdirfile_open, TRIM(dirfilename), LEN_TRIM(dirfilename), flags)
end function

subroutine fdirfile_close (dirfile)
  integer, intent(in) :: dirfile

  ! call f77 library
  call gdfcls(dirfile)
end subroutine

subroutine fdirfile_flush (dirfile, field_code)
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code

  ! call f77 library -- this will take care of figuring out whether
  !                     field_code is empty or not
  call gdffls(dirfile, TRIM(field_code), LEN_TRIM(field_code))
end subroutine

! getdata with return_type=GD_NULL
function fgetdata_n (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples)
  integer :: fgetdata_n
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples

  ! call f77 library
  call gdfget(fgetdata_n, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_NULL, 0)
end function

! getdata with return_type=GD_INT8
function fgetdata_i1 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_out)
  integer :: fgetdata_i1
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*1, dimension(:), intent(out) :: data_out

  ! call f77 library
  call gdfget(fgetdata_i1, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT8, data_out)
end function

! getdata with return_type=GD_INT16
function fgetdata_i2 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_out)
  integer :: fgetdata_i2
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*2, dimension(:), intent(out) :: data_out

  ! call f77 library
  call gdfget(fgetdata_i2, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT16, data_out)
end function

! getdata with return_type=GD_INT32
function fgetdata_i4 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_out)
  integer :: fgetdata_i4
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*4, dimension(:), intent(out) :: data_out

  ! call f77 library
  call gdfget(fgetdata_i4, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT32, data_out)
end function

! getdata with return_type=GD_INT64
function fgetdata_i8 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_out)
  integer :: fgetdata_i8
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*8, dimension(:), intent(out) :: data_out

  ! call f77 library
  call gdfget(fgetdata_i8, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT64, data_out)
end function

! getdata with return_type=GD_FLOAT32
function fgetdata_r4 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_out)
  integer :: fgetdata_r4
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  real*4, dimension(:), intent(out) :: data_out

  ! call f77 library
  call gdfget(fgetdata_r4, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_FLOAT32, data_out)
end function

! getdata with return_type=GD_FLOAT64
function fgetdata_r8 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_out)
  integer :: fgetdata_r8
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  real*8, dimension(:), intent(out) :: data_out

  ! call f77 library
  call gdfget(fgetdata_r8, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_FLOAT64, data_out)
end function

!get_constant with return_type=GD_NULL
function fget_constant_n (dirfile, field_code)
  integer :: fget_constant_n
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  
  ! call f77 library
  call gdfgco(fget_constant_n, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_NULL, 0)
end function

!get_constant with return_type=GD_INT8
function fget_constant_i1 (dirfile, field_code, data_out)
  integer :: fget_constant_i1
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*1, intent(out)  :: data_out
  
  ! call f77 library
  call gdfgco(fget_constant_i1, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT8, data_out)
end function

!get_constant with return_type=GD_INT16
function fget_constant_i2 (dirfile, field_code, data_out)
  integer :: fget_constant_i2
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*2, intent(out)  :: data_out
  
  ! call f77 library
  call gdfgco(fget_constant_i2, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT16, data_out)
end function

!get_constant with return_type=GD_INT32
function fget_constant_i4 (dirfile, field_code, data_out)
  integer :: fget_constant_i4
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*4, intent(out)  :: data_out
  
  ! call f77 library
  call gdfgco(fget_constant_i4, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT32, data_out)
end function

!get_constant with return_type=GD_INT64
function fget_constant_i8 (dirfile, field_code, data_out)
  integer :: fget_constant_i8
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*8, intent(out)  :: data_out
  
  ! call f77 library
  call gdfgco(fget_constant_i8, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT64, data_out)
end function

!get_constant with return_type=GD_FLOAT32
function fget_constant_r4 (dirfile, field_code, data_out)
  integer :: fget_constant_r4
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  real*4, intent(out)  :: data_out
  
  ! call f77 library
  call gdfgco(fget_constant_r4, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_FLOAT32, data_out)
end function

!get_constant with return_type=GD_FLOAT64
function fget_constant_r8 (dirfile, field_code, data_out)
  integer :: fget_constant_r8
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  real*8, intent(out)  :: data_out
  
  ! call f77 library
  call gdfgco(fget_constant_r8, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_FLOAT64, data_out)
end function

function fget_string (dirfile, field_code, length, data_out)
  integer :: fget_string
  integer, intent(in) :: dirfile, length
  character (len=*), intent(in) :: field_code
  character (len=*), intent(out) :: data_out

  ! call f77 library
  call gdfgst(fget_string, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  length, data_out)
end function

function fget_nfields (dirfile)
  integer :: fget_nfields
  integer, intent(in) :: dirfile

  ! call f77 library
  call gdfnfd(fget_nfields, dirfile)
end function

function fget_nfields_by_type (dirfile, entype)
  integer :: fget_nfields_by_type
  integer, intent(in) :: dirfile, entype

  call gdfnft(fget_nfields_by_type, dirfile, entype)
end function

function fget_nvectors (dirfile)
  integer :: fget_nvectors
  integer, intent(in) :: dirfile

  call gdfnve(fget_nvectors, dirfile)
end function

function fget_nmfields (dirfile, parent)
  integer :: fget_nmfields
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: parent

  ! call f77 library
  call gdfnfd(fget_nmfields, dirfile, TRIM(parent), LEN_TRIM(parent))
end function

function fget_nmfields_by_type (dirfile, parent, entype)
  integer :: fget_nmfields_by_type
  integer, intent(in) :: dirfile, entype
  character (len=*), intent(in) :: parent

  call gdfnmt(fget_nmfields_by_type, dirfile, TRIM(parent), LEN_TRIM(parent), &
  entype)
end function

function fget_nmvectors (dirfile, parent)
  integer :: fget_nmvectors
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: parent

  call gdfnmv(fget_nmvectors, dirfile, TRIM(parent), LEN_TRIM(parent))
end function

function fget_field_name_max (dirfile)
  integer :: fget_field_name_max
  integer, intent(in) :: dirfile

  ! call f77 library
  call gdffnx(fget_field_name_max, dirfile)
end function

function fget_mfield_name_max (dirfile, parent)
  integer :: fget_mfield_name_max
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: parent

  ! call f77 library
  call gdfmfx(fget_mfield_name_max, dirfile, TRIM(parent), LEN_TRIM(parent))
end function

subroutine fget_field_list (field_list, dirfile, field_len)
  character(len=*), dimension(:), intent(out) :: field_list
  integer, intent(in) :: dirfile
  integer, intent(inout) :: field_len
  integer :: max_len, nfields, i

  ! make sure the field list is large enough
  max_len = fget_field_name_max(dirfile)

  if (field_len .lt. max_len) then
    field_len = max_len
  else
    nfields = fget_nfields(dirfile)
    do i=1,nfields
      ! call f77 library
      call gdffdn(field_list(i), field_len, dirfile, i)
    end do
  end if
end subroutine

subroutine fget_field_list_by_type (field_list, dirfile, entype, field_len)
  character(len=*), dimension(:), intent(out) :: field_list
  integer, intent(in) :: dirfile, entype
  integer, intent(inout) :: field_len
  integer :: max_len, nfields, i

  ! make sure the field list is large enough
  max_len = fget_field_name_max(dirfile)

  if (field_len .lt. max_len) then
    field_len = max_len
  else
    nfields = fget_nfields_by_type(dirfile, entype)
    do i=1,nfields
      ! call f77 library
      call gdffnt(field_list(i), field_len, dirfile, entype, i)
    end do
  end if
end subroutine

subroutine fget_vector_list (field_list, dirfile, field_len)
  character(len=*), dimension(:), intent(out) :: field_list
  integer, intent(in) :: dirfile
  integer, intent(inout) :: field_len
  integer :: max_len, nfields, i

  ! make sure the field list is large enough
  max_len = fget_field_name_max(dirfile)

  if (field_len .lt. max_len) then
    field_len = max_len
  else
    nfields = fget_nvectors(dirfile)
    do i=1,nfields
      ! call f77 library
      call gdfven(field_list(i), field_len, dirfile, i)
    end do
  end if
end subroutine

subroutine fget_mfield_list (field_list, dirfile, parent, field_len)
  character(len=*), dimension(:), intent(out) :: field_list
  integer, intent(in) :: dirfile
  integer, intent(inout) :: field_len
  integer :: max_len, nfields, i
  character (len=*), intent(in) :: parent

  ! make sure the field list is large enough
  max_len = fget_mfield_name_max(dirfile, parent)

  if (field_len .lt. max_len) then
    field_len = max_len
  else
    nfields = fget_nmfields(dirfile, parent)
    do i=1,nfields
      ! call f77 library
      call gdfmfn(field_list(i), field_len, dirfile, TRIM(parent), &
      LEN_TRIM(parent), i)
    end do
  end if
end subroutine

subroutine fget_mfield_list_by_type (field_list, dirfile, parent, entype, &
field_len)
  character(len=*), dimension(:), intent(out) :: field_list
  integer, intent(in) :: dirfile, entype
  integer, intent(inout) :: field_len
  integer :: max_len, nfields, i
  character (len=*), intent(in) :: parent

  ! make sure the field list is large enough
  max_len = fget_mfield_name_max(dirfile, parent)

  if (field_len .lt. max_len) then
    field_len = max_len
  else
    nfields = fget_nmfields_by_type(dirfile, parent, entype)
    do i=1,nfields
      ! call f77 library
      call gdfmft(field_list(i), field_len, dirfile, TRIM(parent), &
      LEN_TRIM(parent), entype, i)
    end do
  end if
end subroutine

subroutine fget_mvector_list (field_list, dirfile, parent, field_len)
  character(len=*), dimension(:), intent(out) :: field_list
  integer, intent(in) :: dirfile
  integer, intent(inout) :: field_len
  integer :: max_len, nfields, i
  character (len=*), intent(in) :: parent

  ! make sure the field list is large enough
  max_len = fget_mfield_name_max(dirfile, parent)

  if (field_len .lt. max_len) then
    field_len = max_len
  else
    nfields = fget_nmvectors(dirfile, parent)
    do i=1,nfields
      ! call f77 library
      call gdfmvn(field_list(i), field_len, TRIM(parent), LEN_TRIM(parent), &
      dirfile, i)
    end do
  end if
end subroutine

function fget_nframes (dirfile)
  integer :: fget_nframes
  integer, intent(in) :: dirfile

  ! call f77 library
  call gdfnfr(fget_nframes, dirfile)
end function

function fget_spf (dirfile, field_code)
  integer :: fget_spf
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code

  ! call f77 library
  call gdfspf(fget_spf, dirfile, TRIM(field_code), LEN_TRIM(field_code))
end function

! putdata with data_type=GD_INT8
function fputdata_i1 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_in)
  integer :: fputdata_i1
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*1, dimension(:), intent(in) :: data_in

  ! call f77 library
  call gdfput(fputdata_i1, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT8, data_in)
end function

! putdata with data_type=GD_INT16
function fputdata_i2 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_in)
  integer :: fputdata_i2
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*2, dimension(:), intent(in) :: data_in

  ! call f77 library
  call gdfput(fputdata_i2, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT16, data_in)
end function

! putdata with data_type=GD_INT32
function fputdata_i4 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_in)
  integer :: fputdata_i4
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*4, dimension(:), intent(in) :: data_in

  ! call f77 library
  call gdfput(fputdata_i4, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT32, data_in)
end function

! putdata with data_type=GD_INT64
function fputdata_i8 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_in)
  integer :: fputdata_i8
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  integer*8, dimension(:), intent(in) :: data_in

  ! call f77 library
  call gdfput(fputdata_i8, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_INT64, data_in)
end function

! putdata with data_type=GD_FLOAT32
function fputdata_r4 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_in)
  integer :: fputdata_r4
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  real*4, dimension(:), intent(in) :: data_in

  ! call f77 library
  call gdfput(fputdata_r4, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_FLOAT32, data_in)
end function

! putdata with data_type=GD_FLOAT64
function fputdata_r8 (dirfile, field_code, first_frame, first_sample, &
  num_frames, num_samples, data_in)
  integer :: fputdata_r8
  integer, intent(in) :: dirfile, first_frame, first_sample, num_frames
  character (len=*), intent(in) :: field_code
  integer, intent(in) :: num_samples
  real*8, dimension(:), intent(in) :: data_in

  ! call f77 library
  call gdfput(fputdata_r8, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  first_frame, first_sample, num_frames, num_samples, GD_FLOAT64, data_in)
end function

! put_constant with data_type=GD_INT8
function fput_constant_i1 (dirfile, field_code, data_in)
  integer :: fput_constant_i1
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*1, intent(in) :: data_in

  ! call f77 library
  call gdfpco(fput_constant_i1, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT8, data_in)
end function

! put_constant with data_type=GD_INT16
function fput_constant_i2 (dirfile, field_code, data_in)
  integer :: fput_constant_i2
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*2, intent(in) :: data_in

  ! call f77 library
  call gdfpco(fput_constant_i2, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT16, data_in)
end function

! put_constant with data_type=GD_INT32
function fput_constant_i4 (dirfile, field_code, data_in)
  integer :: fput_constant_i4
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*4, intent(in) :: data_in

  ! call f77 library
  call gdfpco(fput_constant_i4, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT32, data_in)
end function

! put_constant with data_type=GD_INT64
function fput_constant_i8 (dirfile, field_code, data_in)
  integer :: fput_constant_i8
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  integer*8, intent(in) :: data_in

  ! call f77 library
  call gdfpco(fput_constant_i8, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_INT64, data_in)
end function

! put_constant with data_type=GD_FLOAT32
function fput_constant_r4 (dirfile, field_code, data_in)
  integer :: fput_constant_r4
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  real*4, intent(in) :: data_in

  ! call f77 library
  call gdfpco(fput_constant_r4, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_FLOAT32, data_in)
end function

! put_constant with data_type=GD_FLOAT64
function fput_constant_r8 (dirfile, field_code, data_in)
  integer :: fput_constant_r8
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  real*8, intent(in) :: data_in

  ! call f77 library
  call gdfpco(fput_constant_r8, dirfile, TRIM(field_code), &
  LEN_TRIM(field_code), GD_FLOAT64, data_in)
end function

function fput_string (dirfile, field_code, data_in)
  integer :: fput_string
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code, data_in

  call gdfpst(fput_string, dirfile, TRIM(field_code), LEN_TRIM(field_code), &
  LEN_TRIM(data_in), TRIM(data_in))
end function

function fget_error (dirfile)
  integer :: fget_error
  integer, intent(in) :: dirfile

  ! call f77 library
  call gdferr(fget_error, dirfile)
end function

subroutine fget_error_string (dirfile, buffer, len)
  integer, intent(in) :: dirfile, len
  character (len=*), intent(out) :: buffer

  ! call f77 library
  call gdfstr(dirfile, buffer, len)
end subroutine

function fget_entry (dirfile, field_code, ent)
  integer :: fget_entry
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  type(gd_entry), intent(out) :: ent

  ! the call may modify the lengths, so don't use parameters
  integer :: len1 = GD_FIELD_LEN
  integer :: len2 = GD_FIELD_LEN
  integer :: len3 = GD_FIELD_LEN

  ! get field type from f77 library
  call gdffdt (fget_entry, dirfile, TRIM(field_code), LEN_TRIM(field_code))

  if (fget_entry .EQ. GD_RAW_ENTRY) then
    ! raw
    call gdferw(ent%spf, ent%data_type, ent%fragment_index, dirfile, &
    TRIM(field_code), LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_LINCOM_ENTRY) then
    ! lincom
    call gdfelc(ent%n_fields, ent%field(1), len1, ent%m(1), ent%b(1), &
    ent%field(2), len2, ent%m(2), ent%b(2), ent%field(3), len3, ent%m(3), &
    ent%b(3), ent%fragment_index, dirfile, TRIM(field_code), &
    LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_LINTERP_ENTRY) then
    ! linterp
    call gdfelt(ent%field(1), len1, ent%field(2), len2, ent%fragment_index, &
    dirfile, TRIM(field_code), LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_BIT_ENTRY) then
    ! bit
    call gdfebt(ent%field(1), len1, ent%bitnum, ent%numbits, &
    ent%fragment_index, dirfile, TRIM(field_code), LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_MULTIPLY_ENTRY) then
    ! multiply
    call gdfemt(ent%field(1), len1, ent%field(2), len2, ent%fragment_index, &
    dirfile, TRIM(field_code), LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_PHASE_ENTRY) then
    !phase
    call gdfemt(ent%field(1), len1, ent%shift, ent%fragment_index, dirfile, &
    TRIM(field_code), LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_CONST_ENTRY) then
    !const
    call gdfeco(ent%data_type, ent%fragment_index, dirfile, TRIM(field_code), &
    LEN_TRIM(field_code))
  else if (fget_entry .EQ. GD_STRING_ENTRY) then
    !string
    call gdffgi(ent%fragment_index, dirfile, TRIM(field_code), &
    LEN_TRIM(field_code))
  else
    ! on the off chance we got a type we weren't prepared to deal with
    fget_entry = GD_NO_ENTRY
  end if
  ent%field_type = fget_entry
end function

subroutine fdirfile_add (dirfile, field_code, ent)
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code
  type(gd_entry), intent(in) :: ent
  integer*4 :: zero = 0
  character (len=1) :: nil = ""

  if (ent%field_type .EQ. GD_RAW_ENTRY) then
    call gdfarw(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    ent%data_type, ent%spf, ent%fragment_index)
  else if (ent%field_type .EQ. GD_LINCOM_ENTRY) then
    call gdfalc(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    ent%n_fields, TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), ent%m(1), &
    ent%b(1), TRIM(ent%field(2)), LEN_TRIM(ent%field(2)), ent%m(2), ent%b(2), &
    TRIM(ent%field(3)), LEN_TRIM(ent%field(3)), ent%m(3), ent%b(3), &
    ent%fragment_index)
  else if (ent%field_type .EQ. GD_LINTERP_ENTRY) then
    call gdfalt(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), TRIM(ent%field(2)), &
    LEN_TRIM(ent%field(2)), ent%fragment_index)
  else if (ent%field_type .EQ. GD_BIT_ENTRY) then
    call gdfabt(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), ent%bitnum, ent%numbits, &
    ent%fragment_index)
  else if (ent%field_type .EQ. GD_MULTIPLY_ENTRY) then
    call gdfamt(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), TRIM(ent%field(2)), &
    LEN_TRIM(ent%field(2)), ent%fragment_index)
  else if (ent%field_type .EQ. GD_PHASE_ENTRY) then
    call gdfaph(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), ent%shift, ent%fragment_index)
  else if (ent%field_type .EQ. GD_CONST_ENTRY) then
    call gdfaco(dirfile, TRIM(field_code), LEN_TRIM(field_code), &
    ent%data_type, GD_INT32, zero, ent%fragment_index)
  else if (ent%field_type .EQ. GD_STRING_ENTRY) then
    call gdfast(dirfile, TRIM(field_code), LEN_TRIM(field_code), nil, zero, &
    ent%fragment_index);
  end if
end subroutine

subroutine fdirfile_madd (dirfile, parent, field_code, ent)
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: field_code, parent
  type(gd_entry), intent(in) :: ent
  integer*4 :: zero = 0
  character (len=1) :: nil = ""

  if (ent%field_type .EQ. GD_LINCOM_ENTRY) then
    call gdfmlc(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), ent%n_fields, TRIM(ent%field(1)), &
    LEN_TRIM(ent%field(1)), ent%m(1), ent%b(1), TRIM(ent%field(2)), &
    LEN_TRIM(ent%field(2)), ent%m(2), ent%b(2), TRIM(ent%field(3)), &
    LEN_TRIM(ent%field(3)), ent%m(3), ent%b(3))
  else if (ent%field_type .EQ. GD_LINTERP_ENTRY) then
    call gdfmlt(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), &
    TRIM(ent%field(2)), LEN_TRIM(ent%field(2)))
  else if (ent%field_type .EQ. GD_BIT_ENTRY) then
    call gdfmbt(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), &
    ent%bitnum, ent%numbits)
  else if (ent%field_type .EQ. GD_MULTIPLY_ENTRY) then
    call gdfmmt(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), &
    TRIM(ent%field(2)), LEN_TRIM(ent%field(2)))
  else if (ent%field_type .EQ. GD_PHASE_ENTRY) then
    call gdfmph(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), TRIM(ent%field(1)), LEN_TRIM(ent%field(1)), ent%shift)
  else if (ent%field_type .EQ. GD_CONST_ENTRY) then
    call gdfmco(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), ent%data_type, GD_INT32, zero)
  else if (ent%field_type .EQ. GD_STRING_ENTRY) then
    call gdfmst(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(field_code), &
    LEN_TRIM(field_code), nil, zero)
  end if
end subroutine

subroutine fdirfile_add_spec (dirfile, spec, fragment_index)
  integer, intent(in) :: dirfile, fragment_index
  character (len=*), intent(in) :: spec

  call gdfasp(dirfile, fragment_index, TRIM(spec), LEN_TRIM(spec))
end subroutine

subroutine fdirfile_madd_spec (dirfile, spec, parent)
  integer, intent(in) :: dirfile
  character (len=*), intent(in) :: parent, spec

  call gdfmsp(dirfile, TRIM(parent), LEN_TRIM(parent), TRIM(spec), &
  LEN_TRIM(spec))
end subroutine

function fget_fragmentname (dirfile, ind)
  character (len=GD_FIELD_LEN) :: fget_fragmentname
  integer, intent(in) :: dirfile, ind
  integer :: l = 4096

  call gdffgn(fget_fragmentname, l, dirfile, ind)
end function

function fget_nfragments (dirfile)
  integer :: fget_nfragments
  integer, intent(in) :: dirfile

  call gdfnfg(fget_nfragments, dirfile)
end function

subroutine fdirfile_metaflush (dirfile)
  integer, intent(in) :: dirfile

  call gdfmfl(dirfile)
end subroutine

subroutine fdirfile_include (dirfile, fragmentname, fragment_index, flags)
  integer, intent(in) :: dirfile, fragment_index, flags
  character (len=*), intent(in) :: fragmentname

  call gdfinc(dirfile, TRIM(fragmentname), LEN_TRIM(fragmentname), &
  fragment_index, flags)
end subroutine
end module
