/* (C) 2008 D. V. Wiebe
 *
 ***************************************************************************
 *
 * This file is part of the GetData project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GetData is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with GetData; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "internal.h"

#ifdef STDC_HEADERS
#include <stdlib.h>
#endif

/* _GD_FreeD: free the DIRFILE and its subordinates
*/
static void _GD_FreeD(DIRFILE* D)
{
  unsigned int i;
  int j;

  dtrace("%p", D);

  for (i = 0; i < D->n_entries; ++i) 
    _GD_FreeE(D->entry[i], 1);

  for (j = 0; j < D->n_fragment; ++j) {
    free(D->fragment[j].cname);
    free(D->fragment[j].sname);
    free(D->fragment[j].ename);
    free(D->fragment[j].ref_name);
  }

  free(D->entry);
  free(D->error_string);
  free(D->error_file);
  free(D->field_list);
  free(D->vector_list);
  for (j = 0; j < GD_N_ENTYPES; ++j)
    free(D->type_list[j]);
  free(D->string_value_list);
  free(D->const_value_list);
  free(D->fragment);
  free(D->name);
  free(D);

  dreturnvoid();
}

/* dirfile_close: Close the specified dirfile and free memory.  This must
 * return an error indicator, since checking D->error after this call won't
 * work if the function was a success.
*/
int dirfile_close(DIRFILE* D)
{
  unsigned int i;

  dtrace("%p", D);

  if (D == NULL) {
    dreturn("%i", 0);
    return 0;
  }

  _GD_ClearError(D);

  /* Flush */
  _GD_FlushMeta(D);
  for(i = 0; i < D->n_entries; ++i)
    if (D->entry[i]->field_type == GD_RAW_ENTRY)
      _GD_Flush(D, D->entry[i], D->entry[i]->field);

  if (D->error) {
    dreturn("%i", 1);
    return -1;
  }

  _GD_FreeD(D);

  dreturn("%i", 0);
  return 0;
}
/* vim: ts=2 sw=2 et tw=80
*/
